@@ -13,10 +13,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import httplib
 import logging
 import socket
-import urlparse
+
+from oslo_utils import encodeutils
+from six.moves import http_client
+from six.moves import urllib
 
 from glance_store import capabilities
 import glance_store.driver
@@ -60,7 +62,7 @@ class StoreLocation(glance_store.location.StoreLocation):
         in the URL are interpreted differently in Python 2.6.1+ than prior
         versions of Python.
         """
-        pieces = urlparse.urlparse(uri)
+        pieces = urllib.parse.urlparse(uri)
         assert pieces.scheme in ('https', 'http')
         self.scheme = pieces.scheme
         netloc = pieces.netloc
@@ -89,6 +91,15 @@ class StoreLocation(glance_store.location.StoreLocation):
         if netloc == '':
             LOG.info(_("No address specified in HTTP URL"))
             raise exceptions.BadStoreUri(uri=uri)
+        else:
+            # IPv6 address has the following format [1223:0:0:..]:<some_port>
+            # we need to be sure that we are validating port in both IPv4,IPv6
+            delimiter = "]:" if netloc.count(":") > 1 else ":"
+            host, dlm, port = netloc.partition(delimiter)
+            # if port is present in location then validate port format
+            if port and not port.isdigit():
+                raise exceptions.BadStoreUri(uri=uri)
+
         self.netloc = netloc
         self.path = path
 
@@ -98,7 +109,7 @@ def http_response_iterator(conn, response, size):
     Return an iterator for a file-like object.
 
     :param conn: HTTP(S) Connection
-    :param response: httplib.HTTPResponse object
+    :param response: http_client.HTTPResponse object
     :param size: Chunk size to iterate with
     """
     chunk = response.read(size)
@@ -122,7 +133,7 @@ class Store(glance_store.driver.Store):
         where to find the image file, and returns a tuple of generator
         (for reading the image file) and image_size
 
-        :param location `glance_store.location.Location` object, supplied
+        :param location: `glance_store.location.Location` object, supplied
                         from glance_store.location.get_location_from_uri()
         """
         try:
@@ -138,7 +149,7 @@ class Store(glance_store.driver.Store):
         class ResponseIndexable(glance_store.Indexable):
             def another(self):
                 try:
-                    return self.wrapped.next()
+                    return next(self.wrapped)
                 except StopIteration:
                     return ''
 
@@ -152,21 +163,16 @@ class Store(glance_store.driver.Store):
         Takes a `glance_store.location.Location` object that indicates
         where to find the image file, and returns the size
 
-        :param location `glance_store.location.Location` object, supplied
+        :param location: `glance_store.location.Location` object, supplied
                         from glance_store.location.get_location_from_uri()
         """
         try:
             size = self._query(location, 'HEAD')[2]
-        except socket.error:
-            reason = _("The HTTP URL is invalid.")
+        except (socket.error, http_client.HTTPException) as exc:
+            err_msg = encodeutils.exception_to_unicode(exc)
+            reason = _("The HTTP URL is invalid: %s") % err_msg
             LOG.info(reason)
             raise exceptions.BadStoreUri(message=reason)
-        except exceptions.NotFound:
-            raise
-        except Exception:
-            # NOTE(flaper87): Catch more granular exceptions,
-            # keeping this branch for backwards compatibility.
-            return 0
         return size
 
     def _query(self, location, verb, depth=0):
@@ -183,7 +189,7 @@ class Store(glance_store.driver.Store):
 
         # Check for bad status codes
         if resp.status >= 400:
-            if resp.status == httplib.NOT_FOUND:
+            if resp.status == http_client.NOT_FOUND:
                 reason = _("HTTP datastore could not find image at URI.")
                 LOG.debug(reason)
                 raise exceptions.NotFound(message=reason)
@@ -218,5 +224,5 @@ class Store(glance_store.driver.Store):
         Returns connection class for accessing the resource. Useful
         for dependency injection and stubouts in testing...
         """
-        return {'http': httplib.HTTPConnection,
-                'https': httplib.HTTPSConnection}[loc.scheme]
+        return {'http': http_client.HTTPConnection,
+                'https': http_client.HTTPSConnection}[loc.scheme]
