@@ -19,7 +19,9 @@ import hashlib
 import logging
 import math
 
+from keystoneclient import service_catalog as keystone_sc
 from oslo_config import cfg
+from oslo_utils import encodeutils
 from oslo_utils import excutils
 from oslo_utils import units
 import six
@@ -33,8 +35,6 @@ except ImportError:
 import glance_store
 from glance_store._drivers.swift import utils as sutils
 from glance_store import capabilities
-from glance_store.common import auth
-from glance_store.common import utils as cutils
 from glance_store import driver
 from glance_store import exceptions
 from glance_store import i18n
@@ -147,8 +147,9 @@ def swift_retry_iter(resp_iter, length, store, location, context):
                 yield chunk
                 bytes_read += len(chunk)
         except swiftclient.ClientException as e:
-            LOG.warn(_("Swift exception raised %s") %
-                     cutils.exception_to_str(e))
+            LOG.warning(_("Swift exception raised %s")
+
+                        % encodeutils.exception_to_unicode(e))
 
         if bytes_read != length:
             if retries == store.conf.glance_store.swift_store_retry_get_count:
@@ -437,7 +438,7 @@ class BaseStore(driver.Store):
         except swiftclient.ClientException as e:
             if e.http_status == http_client.NOT_FOUND:
                 msg = _("Swift could not find object %s.") % location.obj
-                LOG.warn(msg)
+                LOG.warning(msg)
                 raise exceptions.NotFound(message=msg)
             else:
                 raise
@@ -498,7 +499,7 @@ class BaseStore(driver.Store):
 
     @capabilities.check
     def add(self, image_id, image_file, image_size,
-            connection=None, context=None):
+            connection=None, context=None, verifier=None):
         location = self.create_location(image_id, context=context)
         if not connection:
             connection = self.get_connection(location, context=context)
@@ -545,7 +546,11 @@ class BaseStore(driver.Store):
                         content_length = chunk_size
 
                     chunk_name = "%s-%05d" % (location.obj, chunk_id)
-                    reader = ChunkReader(image_file, checksum, chunk_size)
+                    reader = ChunkReader(image_file, checksum, chunk_size,
+                                         verifier)
+                    if reader.is_zero_size is True:
+                        LOG.debug('Not writing zero-length chunk.')
+                        break
                     try:
                         chunk_etag = connection.put_object(
                             location.container, chunk_name, reader,
@@ -572,14 +577,6 @@ class BaseStore(driver.Store):
                             'chunk_etag': chunk_etag})
                     LOG.debug(msg)
 
-                    if bytes_read == 0:
-                        # Delete the last chunk, because it's of zero size.
-                        # This will happen if size == 0.
-                        LOG.debug("Deleting final zero-length chunk")
-                        connection.delete_object(location.container,
-                                                 chunk_name)
-                        break
-
                     chunk_id += 1
                     combined_chunks_size += bytes_read
 
@@ -621,7 +618,8 @@ class BaseStore(driver.Store):
                 raise exceptions.Duplicate(message=msg)
 
             msg = (_(u"Failed to add object to Swift.\n"
-                     "Got error from Swift: %s.") % cutils.exception_to_str(e))
+                     "Got error from Swift: %s.")
+                   % encodeutils.exception_to_unicode(e))
             LOG.error(msg)
             raise glance_store.BackendException(msg)
 
@@ -702,8 +700,8 @@ class BaseStore(driver.Store):
                         connection.put_container(container)
                     except swiftclient.ClientException as e:
                         msg = (_("Failed to add container to Swift.\n"
-                                 "Got error from Swift: %s.") %
-                               cutils.exception_to_str(e))
+                                 "Got error from Swift: %s.")
+                               % encodeutils.exception_to_unicode(e))
                         raise glance_store.BackendException(msg)
                 else:
                     msg = (_("The container %(container)s does not exist in "
@@ -875,9 +873,10 @@ class MultiTenantStore(BaseStore):
                                                    reason=reason)
         self.storage_url = self.conf_endpoint
         if not self.storage_url:
-            self.storage_url = auth.get_endpoint(
-                context.service_catalog, service_type=self.service_type,
-                endpoint_region=self.region, endpoint_type=self.endpoint_type)
+            sc = {'serviceCatalog': context.service_catalog}
+            self.storage_url = keystone_sc.ServiceCatalogV2(sc).url_for(
+                service_type=self.service_type, region_name=self.region,
+                endpoint_type=self.endpoint_type)
 
         if self.storage_url.startswith('http://'):
             self.scheme = 'swift+http'
@@ -939,27 +938,39 @@ class MultiTenantStore(BaseStore):
 
     def get_connection(self, location, context=None):
         return swiftclient.Connection(
-            None, context.user, None,
             preauthurl=location.swift_url,
             preauthtoken=context.auth_token,
-            tenant_name=context.tenant,
-            auth_version='2', insecure=self.insecure,
+            insecure=self.insecure,
             ssl_compression=self.ssl_compression,
             cacert=self.cacert)
 
 
 class ChunkReader(object):
-    def __init__(self, fd, checksum, total):
+    def __init__(self, fd, checksum, total, verifier=None):
         self.fd = fd
         self.checksum = checksum
         self.total = total
+        self.verifier = verifier
         self.bytes_read = 0
+        self.is_zero_size = False
+        self.byteone = fd.read(1)
+        if len(self.byteone) == 0:
+            self.is_zero_size = True
+
+    def do_read(self, i):
+        if self.bytes_read == 0 and i > 0 and self.byteone is not None:
+            return self.byteone + self.fd.read(i - 1)
+        else:
+            return self.fd.read(i)
 
     def read(self, i):
         left = self.total - self.bytes_read
         if i > left:
             i = left
-        result = self.fd.read(i)
+
+        result = self.do_read(i)
         self.bytes_read += len(result)
         self.checksum.update(result)
+        if self.verifier:
+            self.verifier.update(result)
         return result
