@@ -67,21 +67,6 @@ _VMWARE_OPTS = [
                help=_('Password for authenticating with '
                       'VMware ESX/VC server.'),
                secret=True),
-    cfg.StrOpt('vmware_datacenter_path',
-               default='ha-datacenter',
-               help=_('DEPRECATED. Inventory path to a datacenter. '
-                      'If the vmware_server_host specified is an ESX/ESXi, '
-                      'the vmware_datacenter_path is optional. If specified, '
-                      'it should be "ha-datacenter". This option is '
-                      'deprecated in favor of vmware_datastores and will be '
-                      'removed in the Liberty release.'),
-               deprecated_for_removal=True),
-    cfg.StrOpt('vmware_datastore_name',
-               help=_('DEPRECATED. Datastore associated with the datacenter. '
-                      'This option is deprecated in favor of '
-                      'vmware_datastores and will be removed in the Liberty '
-                      'release.'),
-               deprecated_for_removal=True),
     cfg.IntOpt('vmware_api_retry_count',
                default=10,
                help=_('Number of times VMware ESX/VC server API must be '
@@ -102,11 +87,10 @@ _VMWARE_OPTS = [
         help=_(
             'A list of datastores where the image can be stored. This option '
             'may be specified multiple times for specifying multiple '
-            'datastores. Either one of vmware_datastore_name or '
-            'vmware_datastores is required. The datastore name should be '
-            'specified after its datacenter path, seperated by ":". An '
-            'optional weight may be given after the datastore name, seperated '
-            'again by ":". Thus, the required format becomes '
+            'datastores. The datastore name should be specified after its '
+            'datacenter path, seperated by ":". An optional weight may be '
+            'given after the datastore name, seperated again by ":". Thus, '
+            'the required format becomes '
             '<datacenter_path>:<datastore_name>:<optional_weight>. When '
             'adding an image, the datastore with highest weight will be '
             'selected, unless there is not enough free space available in '
@@ -135,15 +119,18 @@ def http_response_iterator(conn, response, size):
 
 class _Reader(object):
 
-    def __init__(self, data):
+    def __init__(self, data, verifier=None):
         self._size = 0
         self.data = data
         self.checksum = hashlib.md5()
+        self.verifier = verifier
 
     def read(self, size=None):
         result = self.data.read(size)
         self._size += len(result)
         self.checksum.update(result)
+        if self.verifier:
+            self.verifier.update(result)
         return result
 
     @property
@@ -153,11 +140,11 @@ class _Reader(object):
 
 class _ChunkReader(_Reader):
 
-    def __init__(self, data, blocksize=8192):
+    def __init__(self, data, verifier=None, blocksize=8192):
         self.blocksize = blocksize
         self.current_chunk = b""
         self.closed = False
-        super(_ChunkReader, self).__init__(data)
+        super(_ChunkReader, self).__init__(data, verifier)
 
     def read(self, size=None):
         ret = b""
@@ -180,6 +167,8 @@ class _ChunkReader(_Reader):
             chunk_len = len(chunk)
             self._size += chunk_len
             self.checksum.update(chunk)
+            if self.verifier:
+                self.verifier.update(chunk)
             if chunk:
                 if six.PY3:
                     size_header = ('%x\r\n' % chunk_len).encode('ascii')
@@ -294,22 +283,6 @@ class Store(glance_store.Store):
             raise exceptions.BadStoreConfiguration(
                 store_name='vmware_datastore', reason=msg)
 
-        if not (self.conf.glance_store.vmware_datastore_name
-                or self.conf.glance_store.vmware_datastores):
-            msg = (_("Specify at least 'vmware_datastore_name' or "
-                     "'vmware_datastores' option"))
-            LOG.error(msg)
-            raise exceptions.BadStoreConfiguration(
-                store_name='vmware_datastore', reason=msg)
-
-        if (self.conf.glance_store.vmware_datastore_name and
-                self.conf.glance_store.vmware_datastores):
-            msg = (_("Specify either 'vmware_datastore_name' or "
-                     "'vmware_datastores' option"))
-            LOG.error(msg)
-            raise exceptions.BadStoreConfiguration(
-                store_name='vmware_datastore', reason=msg)
-
     def configure(self, re_raise_bsc=False):
         self._sanity_check()
         self.scheme = STORE_SCHEME
@@ -412,15 +385,7 @@ class Store(glance_store.Store):
         return ds_map
 
     def configure_add(self):
-        if self.conf.glance_store.vmware_datastores:
-            datastores = self.conf.glance_store.vmware_datastores
-        else:
-            # Backwards compatibility for vmware_datastore_name and
-            # vmware_datacenter_path.
-            datacenter_path = self.conf.glance_store.vmware_datacenter_path
-            datastore_name = self._option_get('vmware_datastore_name')
-            datastores = ['%s:%s:%s' % (datacenter_path, datastore_name, 0)]
-
+        datastores = self._option_get('vmware_datastores')
         self.datastores = self._build_datastore_weighted_map(datastores)
         self.store_image_dir = self.conf.glance_store.vmware_store_image_dir
 
@@ -461,7 +426,8 @@ class Store(glance_store.Store):
             return cookie.name + '=' + cookie.value
 
     @capabilities.check
-    def add(self, image_id, image_file, image_size, context=None):
+    def add(self, image_id, image_file, image_size, context=None,
+            verifier=None):
         """Stores an image file with supplied identifier to the backend
         storage system and returns a tuple containing information
         about the stored image.
@@ -469,9 +435,10 @@ class Store(glance_store.Store):
         :param image_id: The opaque image identifier
         :param image_file: The image data to write, as a file-like object
         :param image_size: The size of the image data to write, in bytes
+        :param verifier: An object used to verify signatures for images
         :retval tuple of URL in backing store, bytes written, checksum
                 and a dictionary with storage system specific information
-        :raises `glance.common.exceptions.Duplicate` if the image already
+        :raises: `glance.common.exceptions.Duplicate` if the image already
                 existed
                 `glance.common.exceptions.UnexpectedStatus` if the upload
                 request returned an unexpected status. The expected responses
@@ -480,13 +447,13 @@ class Store(glance_store.Store):
         ds = self.select_datastore(image_size)
         if image_size > 0:
             headers = {'Content-Length': image_size}
-            image_file = _Reader(image_file)
+            image_file = _Reader(image_file, verifier)
         else:
             # NOTE (arnaud): use chunk encoding when the image is still being
             # generated by the server (ex: stream optimized disks generated by
             # Nova).
             headers = {'Transfer-Encoding': 'chunked'}
-            image_file = _ChunkReader(image_file)
+            image_file = _ChunkReader(image_file, verifier)
         loc = StoreLocation({'scheme': self.scheme,
                              'server_host': self.server_host,
                              'image_dir': self.store_image_dir,
@@ -574,9 +541,9 @@ class Store(glance_store.Store):
         """Takes a `glance_store.location.Location` object that indicates
         where to find the image file to delete
 
-        :location `glance_store.location.Location` object, supplied
+        :param location: `glance_store.location.Location` object, supplied
                   from glance_store.location.get_location_from_uri()
-        :raises NotFound if image does not exist
+        :raises: NotFound if image does not exist
         """
         file_path = '[%s] %s' % (
             location.store_location.datastore_name,
@@ -592,7 +559,7 @@ class Store(glance_store.Store):
             self.session.wait_for_task(delete_task)
         except vexc.FileNotFoundException:
             msg = _('Image file %s not found') % file_path
-            LOG.warn(msg)
+            LOG.warning(msg)
             raise exceptions.NotFound(message=msg)
         except Exception:
             with excutils.save_and_reraise_exception():
