@@ -23,6 +23,7 @@ import tempfile
 import uuid
 
 from oslo_config import cfg
+from oslo_utils import encodeutils
 from oslo_utils import units
 from oslotest import moxstubout
 import requests_mock
@@ -37,8 +38,6 @@ from glance_store._drivers.swift import store as swift
 from glance_store import backend
 from glance_store import BackendException
 from glance_store import capabilities
-from glance_store.common import auth
-from glance_store.common import utils
 from glance_store import exceptions
 from glance_store import location
 from glance_store.tests import base
@@ -59,8 +58,6 @@ SWIFT_CONF = {'swift_store_auth_address': 'localhost:8080',
               'swift_store_container': 'glance',
               'swift_store_user': 'user',
               'swift_store_key': 'key',
-              'swift_store_auth_address': 'localhost:8080',
-              'swift_store_container': 'glance',
               'swift_store_retry_get_count': 1,
               'default_swift_reference': 'ref1'
               }
@@ -474,8 +471,8 @@ class SwiftTests(object):
             self.store.add(str(uuid.uuid4()), image_swift, 0)
         except BackendException as e:
             exception_caught = True
-            self.assertIn("container noexist does not exist "
-                          "in Swift", utils.exception_to_str(e))
+            self.assertIn("container noexist does not exist in Swift",
+                          encodeutils.exception_to_unicode(e))
         self.assertTrue(exception_caught)
         self.assertEqual(SWIFT_PUT_OBJECT_CALLS, 0)
 
@@ -604,10 +601,57 @@ class SwiftTests(object):
             exception_caught = True
             expected_msg = "container %s does not exist in Swift"
             expected_msg = expected_msg % expected_container
-            self.assertIn(expected_msg, utils.exception_to_str(e))
+            self.assertIn(expected_msg, encodeutils.exception_to_unicode(e))
         self.assertTrue(exception_caught)
         self.assertEqual(SWIFT_PUT_OBJECT_CALLS, 0)
 
+    @mock.patch('glance_store._drivers.swift.utils'
+                '.is_multiple_swift_store_accounts_enabled',
+                mock.Mock(return_value=True))
+    def test_add_with_verifier(self):
+        """Test that the verifier is updated when verifier is provided."""
+        swift_size = FIVE_KB
+        base_byte = b"12345678"
+        swift_contents = base_byte * (swift_size // 8)
+        image_id = str(uuid.uuid4())
+        image_swift = six.BytesIO(swift_contents)
+
+        self.store = Store(self.conf)
+        self.store.configure()
+        orig_max_size = self.store.large_object_size
+        orig_temp_size = self.store.large_object_chunk_size
+        custom_size = units.Ki
+        verifier = mock.MagicMock(name='mock_verifier')
+
+        try:
+            self.store.large_object_size = custom_size
+            self.store.large_object_chunk_size = custom_size
+            self.store.add(image_id, image_swift, swift_size,
+                           verifier=verifier)
+        finally:
+            self.store.large_object_chunk_size = orig_temp_size
+            self.store.large_object_size = orig_max_size
+
+        # Confirm verifier update called expected number of times
+        self.assertEqual(verifier.update.call_count,
+                         2 * swift_size / custom_size)
+
+        # define one chunk of the contents
+        swift_contents_piece = base_byte * (custom_size // 8)
+
+        # confirm all expected calls to update have occurred
+        calls = [mock.call(swift_contents_piece),
+                 mock.call(b''),
+                 mock.call(swift_contents_piece),
+                 mock.call(b''),
+                 mock.call(swift_contents_piece),
+                 mock.call(b''),
+                 mock.call(swift_contents_piece),
+                 mock.call(b''),
+                 mock.call(swift_contents_piece),
+                 mock.call(b'')]
+        verifier.update.assert_has_calls(calls)
+
     @mock.patch('glance_store._drivers.swift.utils'
                 '.is_multiple_swift_store_accounts_enabled',
                 mock.Mock(return_value=False))
@@ -626,11 +670,24 @@ class SwiftTests(object):
         self.config(swift_store_container='container')
         self.config(swift_store_create_container_on_put=True)
         self.config(swift_store_multiple_containers_seed=2)
-        fake_get_endpoint = FakeGetEndpoint('https://some_endpoint')
-        self.stubs.Set(auth, 'get_endpoint', fake_get_endpoint)
+        service_catalog = [
+            {
+                'endpoint_links': [],
+                'endpoints': [
+                    {
+                        'adminURL': 'https://some_admin_endpoint',
+                        'region': 'RegionOne',
+                        'internalURL': 'https://some_internal_endpoint',
+                        'publicURL': 'https://some_endpoint',
+                    },
+                ],
+                'type': 'object-store',
+                'name': 'Object Storage Service',
+            }
+        ]
         ctxt = mock.MagicMock(
             user='user', tenant='tenant', auth_token='123',
-            service_catalog={})
+            service_catalog=service_catalog)
         store = swift.MultiTenantStore(self.conf)
         store.configure()
         location, size, checksum, _ = store.add(expected_image_id, image_swift,
@@ -734,11 +791,8 @@ class SwiftTests(object):
         self.assertEqual(expected_location, loc)
         self.assertEqual(expected_swift_size, size)
         self.assertEqual(expected_checksum, checksum)
-        # Expecting 7 calls to put_object -- 5 chunks, a zero chunk which is
-        # then deleted, and the manifest.  Note the difference with above
-        # where the image_size is specified in advance (there's no zero chunk
-        # in that case).
-        self.assertEqual(SWIFT_PUT_OBJECT_CALLS, 7)
+        # Expecting 6 calls to put_object -- 5 chunks, and the manifest.
+        self.assertEqual(SWIFT_PUT_OBJECT_CALLS, 6)
 
         loc = location.get_location_from_uri(expected_location, conf=self.conf)
         (new_image_swift, new_image_size) = self.store.get(loc)
@@ -771,7 +825,6 @@ class SwiftTests(object):
                 capabilities.BitMasks.WRITE_ACCESS)
         except Exception:
             return False
-        return False
 
     def test_no_store_credentials(self):
         """
@@ -853,7 +906,7 @@ class SwiftTests(object):
 
         self.assertEqual(1, mock_del_obj.call_count)
         _, kwargs = mock_del_obj.call_args
-        self.assertEqual(None, kwargs.get('query_string'))
+        self.assertIsNone(kwargs.get('query_string'))
 
     def test_delete_with_reference_params(self):
         """
@@ -1051,10 +1104,10 @@ class TestStoreAuthV3(TestStoreAuthV1):
 
 
 class FakeConnection(object):
-    def __init__(self, authurl, user, key, retries=5, preauthurl=None,
-                 preauthtoken=None, starting_backoff=1, tenant_name=None,
-                 os_options=None, auth_version="1", insecure=False,
-                 ssl_compression=True, cacert=None):
+    def __init__(self, authurl=None, user=None, key=None, retries=5,
+                 preauthurl=None, preauthtoken=None, starting_backoff=1,
+                 tenant_name=None, os_options=None, auth_version="1",
+                 insecure=False, ssl_compression=True, cacert=None):
         if os_options is None:
             os_options = {}
 
@@ -1279,9 +1332,9 @@ class TestMultiTenantStoreConnections(base.StoreBaseTest):
         connection = self.store.get_connection(self.location,
                                                context=self.context)
         self.assertIsNone(connection.authurl)
-        self.assertEqual(connection.auth_version, '2')
-        self.assertEqual(connection.user, 'tenant:user1')
-        self.assertEqual(connection.tenant_name, 'tenant')
+        self.assertEqual(connection.auth_version, '1')
+        self.assertIsNone(connection.user)
+        self.assertIsNone(connection.tenant_name)
         self.assertIsNone(connection.key)
         self.assertEqual(connection.preauthurl, 'https://example.com')
         self.assertEqual(connection.preauthtoken, '0123')
@@ -1301,15 +1354,22 @@ class TestMultiTenantStoreContext(base.StoreBaseTest):
         self.config(**conf)
         self.store.configure()
         self.register_store_schemes(self.store, 'swift')
-        self.service_catalog = [{
-            "name": "Object Storage",
-            "type": "object-store",
-            "endpoints": [{
-                "publicURL": "http://127.0.0.1:0",
-                "region": "region1",
-                "versionId": "1.0",
-            }]
-        }]
+        service_catalog = [
+            {
+                'endpoint_links': [],
+                'endpoints': [
+                    {
+                        'region': 'RegionOne',
+                        'publicURL': 'http://127.0.0.1:0',
+                    },
+                ],
+                'type': 'object-store',
+                'name': 'Object Storage Service',
+            }
+        ]
+        self.ctx = mock.MagicMock(
+            service_catalog=service_catalog, user='tenant:user1',
+            tenant='tenant', auth_token='0123')
         self.addCleanup(self.conf.reset)
 
     @requests_mock.mock()
@@ -1320,12 +1380,8 @@ class TestMultiTenantStoreContext(base.StoreBaseTest):
         store.configure()
         uri = "swift+http://127.0.0.1/glance_123/123"
         loc = location.get_location_from_uri(uri, conf=self.conf)
-        ctx = mock.MagicMock(
-            service_catalog=self.service_catalog, user='tenant:user1',
-            tenant='tenant', auth_token='0123')
-
         m.get("http://127.0.0.1/glance_123/123")
-        store.get(loc, context=ctx)
+        store.get(loc, context=self.ctx)
         self.assertEqual(b'0123', m.last_request.headers['X-Auth-Token'])
 
     @requests_mock.mock()
@@ -1341,30 +1397,14 @@ class TestMultiTenantStoreContext(base.StoreBaseTest):
         store.configure()
         content = b'Some data'
         pseudo_file = six.BytesIO(content)
-        ctx = mock.MagicMock(
-            service_catalog=self.service_catalog, user='tenant:user1',
-            tenant='tenant', auth_token='0123')
         store.add('123', pseudo_file, len(content),
-                  context=ctx)
-
+                  context=self.ctx)
         self.assertEqual(b'0123',
                          head_req.last_request.headers['X-Auth-Token'])
         self.assertEqual(b'0123',
                          put_req.last_request.headers['X-Auth-Token'])
 
 
-class FakeGetEndpoint(object):
-    def __init__(self, response):
-        self.response = response
-
-    def __call__(self, service_catalog, service_type=None,
-                 endpoint_region=None, endpoint_type=None):
-        self.service_type = service_type
-        self.endpoint_region = endpoint_region
-        self.endpoint_type = endpoint_type
-        return self.response
-
-
 class TestCreatingLocations(base.StoreBaseTest):
     _CONF = cfg.CONF
 
@@ -1378,6 +1418,25 @@ class TestCreatingLocations(base.StoreBaseTest):
         moves.reload_module(swift)
         self.addCleanup(self.conf.reset)
 
+        service_catalog = [
+            {
+                'endpoint_links': [],
+                'endpoints': [
+                    {
+                        'adminURL': 'https://some_admin_endpoint',
+                        'region': 'RegionOne',
+                        'internalURL': 'https://some_internal_endpoint',
+                        'publicURL': 'https://some_endpoint',
+                    },
+                ],
+                'type': 'object-store',
+                'name': 'Object Storage Service',
+            }
+        ]
+        self.ctxt = mock.MagicMock(user='user', tenant='tenant',
+                                   auth_token='123',
+                                   service_catalog=service_catalog)
+
     def test_single_tenant_location(self):
         conf = copy.deepcopy(SWIFT_CONF)
         conf['swift_store_container'] = 'container'
@@ -1414,69 +1473,48 @@ class TestCreatingLocations(base.StoreBaseTest):
 
     def test_multi_tenant_location(self):
         self.config(swift_store_container='container')
-        fake_get_endpoint = FakeGetEndpoint('https://some_endpoint')
-        self.stubs.Set(auth, 'get_endpoint', fake_get_endpoint)
-        ctxt = mock.MagicMock(
-            user='user', tenant='tenant', auth_token='123',
-            service_catalog={})
         store = swift.MultiTenantStore(self.conf)
         store.configure()
-        location = store.create_location('image-id', context=ctxt)
+        location = store.create_location('image-id', context=self.ctxt)
         self.assertEqual(location.scheme, 'swift+https')
         self.assertEqual(location.swift_url, 'https://some_endpoint')
         self.assertEqual(location.container, 'container_image-id')
         self.assertEqual(location.obj, 'image-id')
         self.assertIsNone(location.user)
         self.assertIsNone(location.key)
-        self.assertEqual(fake_get_endpoint.service_type, 'object-store')
 
     def test_multi_tenant_location_http(self):
-        fake_get_endpoint = FakeGetEndpoint('http://some_endpoint')
-        self.stubs.Set(auth, 'get_endpoint', fake_get_endpoint)
-        ctxt = mock.MagicMock(
-            user='user', tenant='tenant', auth_token='123',
-            service_catalog={})
         store = swift.MultiTenantStore(self.conf)
         store.configure()
-        location = store.create_location('image-id', context=ctxt)
-        self.assertEqual(location.scheme, 'swift+http')
-        self.assertEqual(location.swift_url, 'http://some_endpoint')
+        self.ctxt.service_catalog[0]['endpoints'][0]['publicURL'] = \
+            'http://some_endpoint'
+        location = store.create_location('image-id', context=self.ctxt)
+        self.assertEqual('swift+http', location.scheme)
+        self.assertEqual('http://some_endpoint', location.swift_url)
 
     def test_multi_tenant_location_with_region(self):
         self.config(swift_store_region='WestCarolina')
-        fake_get_endpoint = FakeGetEndpoint('https://some_endpoint')
-        self.stubs.Set(auth, 'get_endpoint', fake_get_endpoint)
-        ctxt = mock.MagicMock(
-            user='user', tenant='tenant', auth_token='123',
-            service_catalog={})
         store = swift.MultiTenantStore(self.conf)
         store.configure()
-        store._get_endpoint(ctxt)
-        self.assertEqual(fake_get_endpoint.endpoint_region, 'WestCarolina')
+        self.ctxt.service_catalog[0]['endpoints'][0]['region'] = 'WestCarolina'
+        self.assertEqual('https://some_endpoint',
+                         store._get_endpoint(self.ctxt))
 
     def test_multi_tenant_location_custom_service_type(self):
         self.config(swift_store_service_type='toy-store')
-        fake_get_endpoint = FakeGetEndpoint('https://some_endpoint')
-        self.stubs.Set(auth, 'get_endpoint', fake_get_endpoint)
-        ctxt = mock.MagicMock(
-            user='user', tenant='tenant', auth_token='123',
-            service_catalog={})
+        self.ctxt.service_catalog[0]['type'] = 'toy-store'
         store = swift.MultiTenantStore(self.conf)
         store.configure()
-        store._get_endpoint(ctxt)
-        self.assertEqual(fake_get_endpoint.service_type, 'toy-store')
+        store._get_endpoint(self.ctxt)
+        self.assertEqual('https://some_endpoint',
+                         store._get_endpoint(self.ctxt))
 
     def test_multi_tenant_location_custom_endpoint_type(self):
-        self.config(swift_store_endpoint_type='InternalURL')
-        fake_get_endpoint = FakeGetEndpoint('https://some_endpoint')
-        self.stubs.Set(auth, 'get_endpoint', fake_get_endpoint)
-        ctxt = mock.MagicMock(
-            user='user', tenant='tenant', auth_token='123',
-            service_catalog={})
+        self.config(swift_store_endpoint_type='internalURL')
         store = swift.MultiTenantStore(self.conf)
         store.configure()
-        store._get_endpoint(ctxt)
-        self.assertEqual(fake_get_endpoint.endpoint_type, 'InternalURL')
+        self.assertEqual('https://some_internal_endpoint',
+                         store._get_endpoint(self.ctxt))
 
 
 class TestChunkReader(base.StoreBaseTest):
@@ -1503,10 +1541,36 @@ class TestChunkReader(base.StoreBaseTest):
         while True:
             cr = swift.ChunkReader(infile, checksum, CHUNKSIZE)
             chunk = cr.read(CHUNKSIZE)
-            bytes_read += len(chunk)
-            if not chunk:
+            if len(chunk) == 0:
+                self.assertEqual(True, cr.is_zero_size)
                 break
+            bytes_read += len(chunk)
         self.assertEqual(units.Ki, bytes_read)
+        self.assertEqual('fb10c6486390bec8414be90a93dfff3b',
+                         cr.checksum.hexdigest())
+        data_file.close()
+        infile.close()
+
+    def test_read_zero_size_data(self):
+        """
+        Replicate what goes on in the Swift driver with the
+        repeated creation of the ChunkReader object
+        """
+        CHUNKSIZE = 100
+        checksum = hashlib.md5()
+        data_file = tempfile.NamedTemporaryFile()
+        infile = open(data_file.name, 'rb')
+        bytes_read = 0
+        while True:
+            cr = swift.ChunkReader(infile, checksum, CHUNKSIZE)
+            chunk = cr.read(CHUNKSIZE)
+            if len(chunk) == 0:
+                break
+            bytes_read += len(chunk)
+        self.assertEqual(True, cr.is_zero_size)
+        self.assertEqual(0, bytes_read)
+        self.assertEqual('d41d8cd98f00b204e9800998ecf8427e',
+                         cr.checksum.hexdigest())
         data_file.close()
         infile.close()
 
